#!/usr/bin/perl



=head1 NAME

tilethis - configuration helper for larswm

=head1 SYNOPSIS 

tilehis [-option]

   Options:

   -a  add to specified window class [default]
   -r  remove from specified window class

   -t  tiling window class [default]
   -f  floating window class
   -s  sticky window class
   -o  tool window class

   -h  print usage summary
   -v  print version number and exit

=head1 OPTIONS

=over 8

=item B<-a>  

Add an X11 application to a specified window class (default: tiling)

=item B<-r>  

Remove an X11 application from a specified window class

=item B<-t>  

Tiling window class: windows get automatically positioned and resized by
larswm

=item B<-f>  

Floating window class: windows always stay on top of other windows

=item B<-s>  

Sticky window class: windows stay visible on all virtual desktops

=item B<-o>  

Tool window class: windows are simultaneously floating, sticky and tiled around

=item B<-v>  

Print version number and exit

=item B<-h>  

Print usage summary

=back

=head2 Examples

B<tilethis [-at]> 

make an application tiling

B<tilethis -ro>

turn a 'tool' application back into a normal application

=head1 DESCRIPTION

Helper script for configuring B<larswm> <http://www.fnurt.net/larswm/>,
the tiling window manager for X11. 

Simplifies the task of setting up X11 applications for the tiled
subdesktop, as tools or as floating windows under larswm uses xprop to
gather information about the Class and Instance of an X11 application
and writing it into the personal larswm configuration file


=head1 AUTHOR

Florian Cramer <cantsin@zedat.fu-berlin.de>

=cut

use Getopt::Std;


# default operation on larswm setup (either 'add' or 'remove')
$operation = 'add';			
# default window class for the app selected
$window_class = 'dotileclass';		

# location of temporary file for output of xprop command and writing the new .larswmrc
# (anti-locking naming scheme suggested by Randy Bohn)
$tempfile = "/tmp/tilethis_$$";		
# location of larswm configuration file
$larswm_configfile = $ENV{HOME}.'/.larswmrc';

# the following variable indicates what the script has accomplished yet:
# 0 - the lines to be modified in .larswmrc haven't been found yet
# 1 - the lines have been found, but not yet modified
# 2 - the lines have been modified successfully
# NEVER CHANGE THE VALUE below
$working_status = 0;

# parse command line arguments

# help mode (exits the script)
getopts('afhorstv');
if ($opt_v || $opt_h) {
	print "tilethis 1.0 beta\nby Florian Cramer <cantsin\@zedat.fu-berlin.de>\n";
	if ($opt_h) {
		print "Helper script for configuring the larswm window manager\n";
		print "Sets up any running X11 application for tiling -\n";
		print "or for any other larswm window class\n";
		print "\n";
		print "Action:\n";
		print "-a  [default]  add an X11 application to a window class\n";
		print "-r  remove an X11 application from a window class\n";
		print "\n";
		print "Window class:\n";
		print "-t  [default]  tiling\n";
		print "    (i.e. automatically positioned & resized)\n";
		print "-f  floating\n"; 
		print "    (i.e. always staying on top)\n";
		print "-s  sticky\n";
		print "    (i.e. visible on all desktops)\n";
		print "-o  tool\n";
		print "    (i.e. floating, sticky & tiled around)\n";
		print "\n";
		print "Help:\n";
		print "-v  print version number and exit\n";
		print "-h  prints this help message\n";
		print "\n";
		print "Examples:\n";
		print "tilethis [-at]\n"; 
		print "   make an application tiling\n";
		print "tilethis -ro\n";
		print "   turn a 'tool' application back into a normal application\n";
		}
	die "\n";
	}

# determine operation mode
# 'add'    --> adds an app to a larswm window class
# 'remove  --> removes an app from a larswm window class
if    ($opt_a) {$operation = 'add'}
elsif ($opt_r) {$operation = 'remove'}

# determine window class
if ($opt_o) {
	$window_class = 'toolclass';
	%description = (
		"property" 	=> "tool",
		"add" 		=> "set up as a tool",
		"remove"	=> "set up to be no longer a tool",
		);
	}
elsif ($opt_f) {
	$window_class = 'floatclass';
	%description = (
		"property" 	=> "floating",
		"add" 		=> "set up to float",
		"remove"	=> "set up to no longer float",
		);
	}
elsif ($opt_s) {
	$window_class = 'stickyclass';
	%description = (
		"property" 	=> "sticky",
		"add" 		=> "set up to be sticky",
		"remove"	=> "set up to be no longer sticky",
		);
	}
elsif ($opt_t || $window_class) {
	$window_class = 'dotileclass';
	%description = (
		"property" 	=> "tiling",
		"add" 		=> "set up for tiling",
		"remove"	=> "removed from the tiled subdesktop",
		);
	}


# run xprop to gather the Class~Instance string of an app
print 'Click on the application to be ', $description{$operation}, " in larswm...\n";
system ("xprop > $tempfile");
# (security fix as suggested by Andy Bohn)
chmod (0600, $tempfile);

# read & reformat the xprop output
open OUTPUT, "<$tempfile" || die ("Couldn't open temporary file.\n");
while (<OUTPUT>) {
	# get the line we need
	if (/^WM_CLASS/) {
		# reformat line into Class~Instance string with some regular expression magic
		chop;
		s/^[^"]*"([^"]+)"[^"]*"([^"]+).*/\2~\1/;
		# memorize selected Class~Instance string 
		$selected_xappid = $_; 
		}
	}
close(OUTPUT);

if (not $selected_xappid) {&exit ("No X11 application selected.\n")}

# open larswm configuration file, or fix it if necessary
unless (open LARSWMRC_OLD, "< $larswm_configfile") {
	print "Couldn't find your larswm configuration file $larswm_configfile\n";
	print 'Shall I create a new configuration file from the current defaults? ';
	if (<STDIN> =~ /^y/i) {
		system ("larswm -defaults > $larswm_configfile");
		chmod (0600, $larswm_configfile);
		open LARSWMRC_OLD, "< $larswm_configfile" || &exit ("Couldn't create the larswm configuration file.\nI guess that\n- larswm is not properly installed\n- or you are using a pre-7.0.13 version of larswm\n- or your home directory $ENV{HOME} has broken read/write permissions\n- or you typed the 'su' command and work under the wrong user account.\n");
		}
	else {&exit("Please create $larswm_configfile before running this program again.\n")}
	}
# write new larswm configuration file
open LARSWMRC_NEW, ">$tempfile" || &exit ("Couldn't write temporary file\n");

# parse current configuration file
while (<LARSWMRC_OLD>) {
	chop;
	if (/^[\s]*larswm\.$window_class/) {
		# we have found the entries we are looking for
		$working_status = 1;
		# parse the larswm number of the entry 
		$entry_count = $_; $entry_count =~ s/^[\s]*larswm\.$window_class\.([0-9]+).*/\1/;
		# parse the Class~Instance string of the entry
		$entry_xappid = $_; $entry_xappid =~ s/^[^:]+:[\s]+(.*)/\1/;
		if ($operation eq 'add') {
			# store all Class~Instance strings in an array 
			push @xappids, $entry_xappid;
			# since we will only _add_ an entry, we can write out the existing lines unmodified 
			print LARSWMRC_NEW $_, "\n";
			}
		elsif ($operation eq 'remove') {
			if ($selected_xappid ne $entry_xappid) {
				# store all entries except the one we want to remove
				push @xappids, $entry_xappid;
				}
			else {
				# yes, we have found a matching entry and want to remove it 
				$remove_flag = 1
			  	# and so of course we _don't_ store it in @xappids	
				}
			}
		}
	# what to do with lines that don't contain the entries we are looking for:
	else {
		# the entries have already been found and parsed
		if ($working_status == 1) {
			# app must be entered into the selected window class 
			if ($operation eq 'add') {
				if ($entry_count > 63) {
					&exit ("This application cannot be $description{$operation};\nlarswm's limit of 64 $description{$property} applications is exceeded.\n");
					}
				else {
					foreach $x (@xappids) {
						if ($x eq $selected_xappid) {&exit ("$selected_xappid is already $description{$operation} in your larswm configuration file $larswm_configfile\n")}
						}
					$entry_count++;
					# add new window class entry to the file
					print LARSWMRC_NEW 'larswm.',$window_class,'.',$entry_count,': ',$selected_xappid,"\n";
					}
				}
			# app must be removed from selected window class
			elsif ($operation eq 'remove') {
				if (not $remove_flag == 1) {&exit("$selected_xappid is already $description{$operation} in your larswm configuration file $larswm_configfile\n")}
				else {
					# rebuild window class entries without the selected app
					for ($x = 0; $x <= $#xappids; $x++) {	
						print LARSWMRC_NEW 'larswm.',$window_class,'.',$x,': ',$xappids[$x],"\n";
						}
					}
				}
			# we are done with modifying the window class entries and thus can skip this section from now on 
			$working_status = 2;
			}
		# since we're now reading lines we won't modify, write them out as they are. 
		print LARSWMRC_NEW $_,"\n";
		}
	}
	


# The following is a fallback operation when _no_ previous entries for the selected
# window class have been found. 
# This simply means that we either append the missing line to the file, or,
# if the removal of an entry was requested, quit with an error message.

if ($working_status == 0) {
	if ($operation eq 'add') {
		print LARSWMRC_NEW "\n";
		# next line is commented out to avoid clutter in the configuration file
		#print LARSWMRC_NEW "! The following line was added by tilethis\n";
		print LARSWMRC_NEW 'larswm.',$window_class,'.0: ',$selected_xappid,"\n\n";
		# ok, we made it
		$working_status = 2;
		}
	elsif ($operation eq 'remove') {
		&exit("$selected_xappid is already $description{$operation} in your larswm configuration file $larswm_configfile\n");
		}
	}

# now we can close the files
close (LARSWMRC_OLD);
close (LARSWMRC_NEW);

# make sure that we successfully modified the larswm configuration file
if ($working_status == 2) {
	# backup previous configuration file 
	# (use 'cat' instead of 'cp' to avoid trouble if 'cp' defaults to 'cp -i')
	system ("cat $larswm_configfile > $larswm_configfile.bak");
	# put the new configuration file in place 
	system ("mv $tempfile $larswm_configfile");

	print "$selected_xappid has now been $description{$operation}\nin your larswm configuration file $larswm_configfile\n\n";

	print "Shall I restart larswm for the change to take effect? ";
	# only works with larswm 7.0.13 or higher
	if (<STDIN> =~ /^[y]/i) {system ("larsremote restart")} 		
	}
else {
	print "Failed to write new configuration.\nLeaving your larswm configuration untouched.\n";
	}
print "\n";


# subroutines


# we use this instead of 'die' in order not to leave stale files in /tmp 
sub exit {
	unlink $tempfile;
	die ($_[0]);
	}

# eof

